using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Runtime.InteropServices;
using System.IO;

namespace ImageCompare.Components
{
    public class IconGetter
    {
        // SHGetFileInfo֐
        [DllImport("shell32.dll")]
        private static extern IntPtr SHGetFileInfo(
            string pszPath,
            uint dwFileAttributes,
            ref SHFILEINFO psfi,
            uint cbSizeFileInfo,
            uint uFlags
        );

        // SHGetFileInfo֐ŎgptO
        private const uint SHGFI_ICON = 0x100; // ACRE\[X̎擾
        private const uint SHGFI_LARGEICON = 0x0; // 傫ACR
        private const uint SHGFI_SMALLICON = 0x1; // ACR
        private const uint SHGFI_SYSICONINDEX = 0x4000;
        private const uint SHGFI_USEFILEATTRIBUTES = 0x10;

        // SHGetFileInfo֐Ŏgp\
        private struct SHFILEINFO
        {
            public IntPtr hIcon;
            public IntPtr iIcon;
            public uint dwAttributes;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
            public string szDisplayName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
            public string szTypeName;
        };

        public static List<IconInfo> GetIconInfos(List<string> paths)
        {
            List<IconInfo> iconInfos = new List<IconInfo>();
            List<string> extensions = new List<string>();

            bool isExistsExtensions = false;
            bool isGotDirectoryIcon = false;
            FileInfo fi = null;
            string extension;

            foreach (string path in paths)
            {
                fi = new FileInfo(path);

                if (fi.Extension != ".exe")
                {
                    //ʏ̃t@C

                    isExistsExtensions = false;
                    extension = fi.Extension;
                    foreach (string _extension in extensions)
                    {
                        if (_extension == extension)
                        {
                            isExistsExtensions = true;
                            break;
                        }

                        if (!isExistsExtensions)
                        {
                            iconInfos.Add(new IconInfo("", extension, IconGetter.GetIcon(fi.FullName), false));
                        }
                    }
                }
                else if (fi.Exists)
                {
                    //exe
                    iconInfos.Add(new IconInfo(fi.FullName, ".exe", IconGetter.GetIcon(fi.FullName), false));
                }
                else if (!isGotDirectoryIcon)
                {
                    //dir
                    isGotDirectoryIcon = true;
                    iconInfos.Add(new IconInfo("", "", IconGetter.GetIcon(fi.FullName), true));
                }
            }

            return iconInfos;
        }

        public static Image GetIcon(string path)
        {
            FileInfo fi = new FileInfo(path);

            uint option =
                SHGFI_ICON |
                SHGFI_SYSICONINDEX |
                SHGFI_SMALLICON |
                (fi.Exists && fi.Extension != ".exe" ? SHGFI_USEFILEATTRIBUTES : 0)
            ;

            // AvP[VEACR擾
            SHFILEINFO shinfo = new SHFILEINFO();
            IntPtr hSuccess = SHGetFileInfo(path, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo), option);
            if (hSuccess != IntPtr.Zero)
            {
                Icon appIcon = Icon.FromHandle(shinfo.hIcon);

                // sN`[{bNXɃAvP[VEACRZbg
                return appIcon.ToBitmap();
            }

            return null;
        }

        public static Image GetIconFromExtension(string extension)
        {
            uint option =
                SHGFI_ICON |
                SHGFI_SYSICONINDEX |
                SHGFI_SMALLICON |
                (extension != ".exe" ? SHGFI_USEFILEATTRIBUTES : 0)
            ;

            // AvP[VEACR擾
            SHFILEINFO shinfo = new SHFILEINFO();
            IntPtr hSuccess = SHGetFileInfo("*" + extension, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo), option);
            if (hSuccess != IntPtr.Zero)
            {
                Icon appIcon = Icon.FromHandle(shinfo.hIcon);

                // sN`[{bNXɃAvP[VEACRZbg
                return appIcon.ToBitmap();
            }

            return null;
        }
    }

    public class IconInfo
    {
        public string extension;
        public string filename;
        public Image image;
        public bool isDirectory;

        public IconInfo(string filename, string extension, Image image, bool isDirectory)
        {
            this.filename = filename;
            this.extension = extension;
            this.image = image;
            this.isDirectory = isDirectory;
        }
    }
}
