using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;

using Microsoft.VisualBasic.FileIO;

using ImageSimilarityCalculator;

using ImageCompare.Forms;

namespace ImageCompare.Controls
{
    public partial class PageImageCompare_1_x : UserControl
    {
        private FolderSearchForm fsf = new FolderSearchForm();

        private ColorMap cmOrg;

        public PageImageCompare_1_x()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                this.textImageUrl.Text = ofd.FileName;
                this.pictureBox.BackgroundImage = Image.FromFile(ofd.FileName);
                this.cmOrg = ColorMap.Create(ofd.FileName, this.pictureBox.BackgroundImage);
            }
        }

        private void buttonOpenFolder_Click(object sender, EventArgs e)
        {

            if (fsf.ShowDialog() == DialogResult.OK)
            {
                foreach (string path in fsf.filenames)
                {
                    foreach (Object o in this.listBoxFolder.Items)
                    {
                        if (o.ToString() == path)
                        {
                            break;
                        }
                    }

                    this.listBoxFolder.Items.Add(path);
                }
            }
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.button2.Enabled = false;

            this.grid.Rows.Clear();

            string path;

            int rowHeight = 100 ;

            foreach (object o in this.listBoxFolder.Items)
            {
                path = o.ToString();

                string[] filenames = Directory.GetFiles(
                    path, 
                    "*.*",
                    (this.checkBoxSearchRecursive.Checked ? System.IO.SearchOption.AllDirectories : System.IO.SearchOption.TopDirectoryOnly)
                );

                Image image = null;

                foreach (string filename in filenames)
                {
                    if (!".jpg|.png|.gif|.bmp".Contains(new FileInfo(filename).Extension.ToLower()))
                    {
                        continue;
                    }

                    Image thum;
                    try
                    {
                        image = Image.FromFile(filename);
                    }
                    catch (Exception ex)
                    {
                        continue;
                    }
                    thum = new Bitmap(image,
                        ((image.Width < image.Height) ? rowHeight * image.Width / image.Height : rowHeight),
                        ((image.Width > image.Height) ? rowHeight * image.Height / image.Width : rowHeight)
                    ) as Image;
                    image.Dispose();

                    ColorMap cm = ColorMap.Create(filename, thum);
                    ImageSimilarityInfo isi = new ImageSimilarityInfo(this.cmOrg, cm);

                    DataGridViewRow row = grid.Rows[grid.Rows.Add()];
                    row.Height = rowHeight;

                    row.Cells[0].Value = false;

                    DataGridViewImageCell cellImage = row.Cells[1] as DataGridViewImageCell;
                    cellImage.ValueIsIcon = false;
                    cellImage.Value = thum;


                    row.Cells[2].Value = Math.Round(isi.similarity, 1).ToString() + "%";

                    row.Cells[3].Value = filename;
                }

            }

            this.grid.Sort(grid.Columns[2], ListSortDirection.Descending);

            this.button2.Enabled = true; ;
        }

        private void buttonDeleteFolder_Click(object sender, EventArgs e)
        {
            for (int i = this.listBoxFolder.SelectedIndices.Count - 1; i >= 0; i--)
            {
                this.listBoxFolder.Items.RemoveAt(this.listBoxFolder.SelectedIndices[i]);
            }
        }

        private void grid_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex > -1)
            {
                switch (e.ColumnIndex)
                {
                    case 0:
                    case 2:
                    case 3:
                        bool flag = !(bool)this.grid.Rows[e.RowIndex].Cells[0].Value;
                        this.grid.Rows[e.RowIndex].Cells[0].Value = flag;
                        this.grid.Rows[e.RowIndex].Cells[0].Style.BackColor = flag ? Color.Red : Color.White;
                        break;
                }
            }
        }

        private void grid_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex > -1)
            {
                switch (e.ColumnIndex)
                {
                    case 1:
                        Process.Start(this.grid.Rows[e.RowIndex].Cells[3].Value.ToString());
                        break;
                }
            }
        }

        private void pictureBox_DoubleClick(object sender, EventArgs e)
        {
            if (this.textImageUrl.Text != "")
            {
                Process.Start(this.textImageUrl.Text);
            }
        }

        private void grid_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex > -1)
            {
                switch (e.ColumnIndex)
                {
                    case 0:
                        bool flag = !(bool)this.grid.Rows[e.RowIndex].Cells[0].Value;
                        this.grid.Rows[e.RowIndex].Cells[0].Value = flag;
                        this.grid.Rows[e.RowIndex].Cells[0].Style.BackColor = !flag ? Color.Red : Color.White;
                        break;
                }
            }
        }

        private void checkAll_CheckedChanged(object sender, EventArgs e)
        {
            foreach (DataGridViewRow row in this.grid.Rows)
            {
                row.Cells[0].Value = this.checkAll.Checked;
                row.Cells[0].Style.BackColor = this.checkAll.Checked ? Color.Red : Color.White;
            }
        }

        private void buttonDeleteImages_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("I摜폜܂B낵łH", "mF", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                for (int i = grid.Rows.Count - 1; i >= 0; i--)
                {
                    if ((bool)grid.Rows[i].Cells[0].Value == true)
                    {
                        FileSystem.DeleteFile(
                            this.grid.Rows[i].Cells[3].Value.ToString(),
                            UIOption.OnlyErrorDialogs,
                            RecycleOption.SendToRecycleBin
                        );
                        this.grid.Rows.RemoveAt(i);
                    }
                }
            }
        }
    }
}
