using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

using ImageCompare.Components;

namespace ImageCompare.Forms
{
    public partial class FolderSearchForm : Form
    {
        public string filename;
        public string[] filenames;

        ImageList smallImageList = new ImageList();
        private bool isExistsDirIconInfo = false;

        public FolderSearchForm()
        {
            InitializeComponent();

            this.smallImageList = new ImageList();
            this.smallImageList.ColorDepth = ColorDepth.Depth32Bit;

            this.listView.SmallImageList = this.smallImageList;

            if (File.Exists(Common.Global.FileName.CurrentSearchPath))
            {
                this.textDestFolder.Text = File.ReadAllText(Common.Global.FileName.CurrentSearchPath);
            }
            else
            {
                File.WriteAllText(Common.Global.FileName.CurrentSearchPath, "");
            }
        }

        private void UpdateSmallImageList(List<string> paths)
        {
            string extension;

            foreach (string path in paths)
            {
                FileInfo fi = new FileInfo(path);
                extension = fi.Extension;

                string key = "";
                Image image = null;

                if (fi.Exists)
                {
                    //ʏ̃t@C

                    key = (fi.Extension.ToLower() == ".exe" ? path : fi.Extension);

                    if (!this.smallImageList.Images.Keys.Contains(key))
                    {
                        image = (fi.Extension == ".exe"
                            ? IconGetter.GetIcon(fi.FullName)
                            : IconGetter.GetIconFromExtension(extension)
                        );
                    }
                    else
                    {
                        continue;
                    }
                }
                else
                {
                    if (!isExistsDirIconInfo)
                    {
                        this.isExistsDirIconInfo = true;
                        key = "<dir>";
                        image = IconGetter.GetIcon(fi.FullName);
                    }
                }

                if (key != "")
                {
                    this.smallImageList.Images.Add(key, image);                    
                }
            }
        }

        private void SaveDestFolder()
        {
            File.WriteAllText(Common.Global.FileName.CurrentSearchPath, this.textDestFolder.Text);
        }

        private void Search()
        {
            List<string> directories = new List<string>(Directory.GetDirectories(this.textDestFolder.Text));
            directories.AddRange(Directory.GetFiles(this.textDestFolder.Text));

            this.UpdateSmallImageList(directories);

            Regex regex = new Regex(this.textSearchWord.Text);

            FileInfo fi = null;

            this.listView.Items.Clear();
            foreach (string dir in directories)
            {
                fi = new FileInfo(dir);

                if (regex.IsMatch(dir))
                {
                    ListViewItem item = new ListViewItem(new DirectoryInfo(dir).Name);
                    item.Tag = dir;
                    item.ImageKey = (fi.Exists ? (fi.Extension == "exe" ? dir : fi.Extension) : "<dir>");

                    this.listView.Items.Add(item);
                }
            }

            if (this.listView.Items.Count == 1)
            {
            }
        }

        private void buttonGetDestFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                this.textDestFolder.Text = dialog.SelectedPath;
                File.WriteAllText(Common.Global.FileName.CurrentSearchPath, dialog.SelectedPath);
            }
        }

        private void buttonSearch_Click(object sender, EventArgs e)
        {
            File.WriteAllText(Common.Global.FileName.CurrentSearchPath, this.textDestFolder.Text);

            this.buttonSearch.Enabled = false;
            this.Search();
            this.buttonSearch.Enabled = true;
        }

        private void listView_SelectedIndexChanged(object sender, EventArgs e)
        {
        }

        private void listView_Click(object sender, EventArgs e)
        {
            
        }

        private void listView_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if (e.IsSelected)
            {
                FileInfo fi = new FileInfo((string)e.Item.Tag);
                if (fi.Exists)
                {
                }
                else
                {
                }
            }
        }

        private void FolderSearchForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing)
            {
                this.Visible = false;
                e.Cancel = true;
            }
        }

        private void textSearchWord_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Enter:
                    this.Search();
                    break;
            }
        }

        private void textSearchWord_Enter(object sender, EventArgs e)
        {
            this.ImeMode = ImeMode.Hiragana;
            this.textSearchWord.SelectAll();
        }

        private void ɍőOʂɕ\ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.TopMost = !this.TopMost;
            this.ɍőOʂɕ\ToolStripMenuItem.Checked = this.TopMost;
        }

        private void textSearchWord_Click(object sender, EventArgs e)
        {
            if (this.textSearchWord.SelectedText == "")
            {
                this.textSearchWord.SelectAll();
            }

        }


        protected override void WndProc(ref Message m)
        {
            base.WndProc(ref m);

        }

        private void FolderSearchForm_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Back | Keys.LButton:
                    //this.GetNextControl(this.ActiveControl, true).Focus();
                    
                    break;

            }
        }

        private void buttonSelectFolder_Click(object sender, EventArgs e)
        {
            if (this.textDestFolder.Text != "")
            {
                this.filename = this.textDestFolder.Text;
                this.filenames = new string[1] { this.textDestFolder.Text };
                this.DialogResult = DialogResult.OK;
            }
        }

        private void buttonSelectItems_Click(object sender, EventArgs e)
        {
            this.SelectItems();
        }

        private void listView_DoubleClick(object sender, EventArgs e)
        {
            this.SelectItems();
        }

        private void SelectItems()
        {

            if (this.listView.SelectedItems.Count > 0)
            {
                List<string> filelist = new List<string>();
                foreach (ListViewItem item in this.listView.SelectedItems)
                {
                    filelist.Add(item.Tag.ToString());
                }
                this.filenames = filelist.ToArray();
                this.filename = filelist[0];

                this.DialogResult = DialogResult.OK;
            }
        }
    }
}