using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ImageSimilarityCalculator
{
    public class ColorMap
    {
        public static Dictionary<string, ColorMap> cache = new Dictionary<string, ColorMap>();

        public static bool enableCache = true;

        public static ColorMap Create(string url)
        {
            using (Bitmap b = new Bitmap(url))
            {
                return ColorMap.Create(url, b);
            }
        }

        public static ColorMap Create(Image i)
        {
            return ColorMap.Create("", new Bitmap(i));
        }

        public static ColorMap Create(Bitmap b)
        {
            return ColorMap.Create("", b);
        }

        public static ColorMap Create(string name, Image i)
        {
            return ColorMap.Create(name, new Bitmap(i));
        }

        public static ColorMap Create(string name, Bitmap b)
        {
            //LbVL΂
            if (ColorMap.enableCache
                && name != ""
                && ColorMap.cache.ContainsKey(name))
            {
                return ColorMap.cache[name];
            }

            return new ColorMap(name, b);
        }


        public string name = "";

        /// <summary>
        /// red per image
        /// </summary>
        public double r = 0;

        /// <summary>
        /// green per image
        /// </summary>
        public double g = 0;

        /// <summary>
        /// blue per image
        /// </summary>
        public double b = 0;

        /// <summary>
        /// monochrome per image
        /// </summary>
        public double m = 0;

        /// <summary>
        /// number of red
        /// </summary>
        public double countR = 0;

        /// <summary>
        /// number of green
        /// </summary>
        public double countG = 0;

        /// <summary>
        /// number of blue
        /// </summary>
        public double countB = 0;

        /// <summary>
        /// number of monochrome
        /// </summary>
        public double countM = 0;

        public int hsvStep = 10;

        /// <summary>
        /// values per image
        /// </summary>
        public double[] v;

        /// <summary>
        /// hues per image
        /// </summary>
        public double[] h;

        /// <summary>
        /// saturations per image
        /// </summary>
        public double[] s;

        /// <summary>
        /// numbers of value
        /// </summary>
        public double[] countV;

        /// <summary>
        /// numbers of hue
        /// </summary>
        public double[] countH;

        /// <summary>
        /// numbers of saturations
        /// </summary>
        public double[] countS;

        /// <summary>
        /// use ColorMap.Create
        /// </summary>
        /// <param name="name"></param>
        /// <param name="b"></param>
        private ColorMap(string name, Bitmap b)
        {
            this.name = name;

            //HSBz񏉊

            this.v = new double[this.hsvStep];
            this.h = new double[this.hsvStep];
            this.s = new double[this.hsvStep];

            this.countV = new double[this.hsvStep];
            this.countH = new double[this.hsvStep];
            this.countS = new double[this.hsvStep];

            for (int i = 0; i < this.hsvStep; i++)
            {
                this.v[i] = 0;
                this.h[i] = 0;
                this.s[i] = 0;

                this.countV[i] = 0;
                this.countH[i] = 0;
                this.countS[i] = 0;
            }

            //pixel

            Color c;

            for (int y = 0; y < b.Height; y++)
            {
                for (int x = 0; x < b.Width; x++)
                {
                    c = b.GetPixel(x, y);

                    //RGBJEg

                    if (c.R > c.G && c.R > c.B) this.countR++;
                    else if (c.G > c.B) countG++;
                    else if (c.G != c.B) countB++;
                    else countM++;

                    //HSBU蕪JEg

                    countH[(int)Math.Round((c.GetHue()) / 360 * (this.hsvStep - 1))]++;
                    countV[(int)Math.Round(c.GetSaturation() * (this.hsvStep - 1))]++;
                    countS[(int)Math.Round(c.GetBrightness() * (this.hsvStep - 1))]++;
                }
            }

            //p[Ze[WvZ

            int countPixel = b.Width * b.Height;

            this.r = this.countR * 100 / countPixel;
            this.g = this.countG * 100 / countPixel;
            this.b = this.countB * 100 / countPixel;
            this.m = this.countM * 100 / countPixel;

            for (int i = 0; i < this.hsvStep; i++)
            {
                this.h[i] = this.countH[i] * 100 / countPixel;
                this.s[i] = this.countS[i] * 100 / countPixel;
                this.v[i] = this.countV[i] * 100 / countPixel;
            }

            if (ColorMap.enableCache
                && this.name != "")
            {
                ColorMap.cache[this.name] = this;
            }
        }

        /*
        private ColorMap(string name, Image i)
        {
            this._ColorMap(name, new Bitmap(i));
        }

        private ColorMap(Bitmap b)
        {
            this._ColorMap("", b);
        }

        private ColorMap(Image i)
        {
            this._ColorMap("", new Bitmap(i));
        }

        private ColorMap(string url)
        {
            using (Bitmap b = new Bitmap(url))
            {
                this._ColorMap(url, b);
            }
        }
        */

    }
}
