using System;
using System.Collections.Generic;
using System.Text;

namespace ImageSimilarityCalculator
{
    public class ImageSimilarityInfo
    {
        ColorMap cm1;
        ColorMap cm2;

        public double similarity;
        
        public double r;
        public double g;
        public double b;
        public double m;

        public int hsvStep = 0;

        public double h;
        public double s;
        public double v;

        public double[] hList;
        public double[] sList;
        public double[] vList;

        public ImageSimilarityInfo(ColorMap cm1, ColorMap cm2)
        {
            this.cm1 = cm1;
            this.cm2 = cm2;

            this.r = cm1.r < cm2.r ? cm1.r * 100 / cm2.r : cm2.r * 100 / cm1.r;
            this.g = cm1.g < cm2.g ? cm1.g * 100 / cm2.g : cm2.g * 100 / cm1.g;
            this.b = cm1.b < cm2.b ? cm1.b * 100 / cm2.b : cm2.b * 100 / cm1.b;
            this.m = cm1.m < cm2.m ? cm1.m * 100 / cm2.m : cm2.m * 100 / cm1.m;

            if (this.cm1.hsvStep == this.cm2.hsvStep)
            {
                this.hsvStep = this.cm1.hsvStep;

                this.hList = new double[this.cm1.hsvStep];
                this.sList = new double[this.cm1.hsvStep];
                this.vList = new double[this.cm1.hsvStep];

                for (int i = 0; i < this.hsvStep; i++)
                {
                    this.hList[i] = this.cm1.h[i] < this.cm2.h[i]
                        ? this.cm1.h[i] * 100 / this.cm2.h[i]
                        : this.cm2.h[i] * 100 / this.cm1.h[i];
                    this.sList[i] = this.cm1.s[i] < this.cm2.s[i]
                        ? this.cm1.s[i] * 100 / this.cm2.s[i]
                        : this.cm2.s[i] * 100 / this.cm1.s[i];
                    this.vList[i] = this.cm1.v[i] < this.cm2.v[i]
                        ? this.cm1.v[i] * 100 / this.cm2.v[i]
                        : this.cm2.v[i] * 100 / this.cm1.v[i];
                }

                double totalH = 0;
                double totalS = 0;
                double totalV = 0;

                for (int i = 0; i < this.hsvStep; i++)
                {
                    totalH += this.hList[i];
                    totalS += this.sList[i];
                    totalV += this.vList[i];
                }

                this.h = totalH / this.hsvStep;
                this.s = totalS / this.hsvStep;
                this.v = totalV / this.hsvStep;
            }

            this.similarity 
                = (this.r + this.g + this.b + this.m + this.h + this.s + this.v) / 7;
        }
    }
}
